#! /bin/sh
# vim:sts=2 sw=2 ts=2 et ai foldmethod=marker tw=118

bn0=`basename ${0}`
dn0=`dirname ${0}`
[ "${dn0}" = "." ] && dn0=`pwd`
agentDir="${dn0}"

if [ -f "${dn0}/.manifest" -o -d "${dn0}/.installer" ]; then #{{{
    cat <<EOS
    Cannot run ${bn0} from the install package. Please run it from
    the installed Agent directory.
EOS
    exit 1
fi #}}}

TermInit() #{{{ Initialize the terminal functions
{
  #{{{
  export b u s i n bu bs su nb ns nu nbu nbs nsu
  b=
  u=
  s=
  i=
  n=
  bu=
  bs=
  su=
  nb=
  ns=
  nu=
  nbu=
  nbs=
  nsu=
  i=
  #}}}

  type tput > /dev/null 2>&1
  [ ${?} -ne 0 ] && return

  #{{{
  b=`tput bold` || b=
  u=`tput smul` || u=
  s=`tput smso` || s=

  bu=${b}${u}
  bs=${b}${s}
  su=${s}${u}
  nb=${n}${b}
  ns=${n}${s}
  nu=${n}${u}
  nbu=${n}${bu}
  nbs=${n}${bs}
  nsu=${n}${su}
  #}}}

  i=`tput setab 0` || i=
  if [ ! -z "${i}" ]; then
    i="${i}`tput setaf 0`" || i=
  fi
  n=`tput sgr0` || n=
} #}}}

LogMessage() #{{{
{
  if [ -z "${logfile}" ]; then
    [ ${quiet} -ne 0 ] && return
    [ ${#} -gt 0 ] && echo "${@}" || cat -
  else
    if [ ${quiet} -eq 0 ]; then
      [ ${#} -gt 0 ] && echo "${@}" | tee -a "${logfile}" || cat - | tee -a "${logfile}"
    else
      [ ${#} -gt 0 ] && echo "${@}" >> "${logfile}" || cat - >> "${logfile}"
    fi
  fi
} #}}}

Error() #{{{
{
  LogMessage "${b}${bn0}${n}: ${@}"
} #}}}

DownCase() #{{{
{
  echo ${1} | tr 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' 'abcdefghijklmnopqrstuvwxyz'
} #}}}

StopAgent() #{{{
{
  command="${init_dir}/vvagent stop quiet"
  LogMessage "Stopping Agent: ${command}"
  sh -c "${command}" 2>/dev/null
} #}}}

RequestShutdown() #{{{
{
  stop_agent=0
  if [ "${1}" = "${progVV}" ]; then #{{{
    FailInstall ${INSTALL_FAILED} "A backup job is currently running. Terminate it and run uninstall.sh again."
  else
    if [ ${force_stop_agent} -eq 1 ]; then #{{{
      stop_agent=1
    else
      printf '%s is still running. Do you wish to stop it? ([Y]/n) ' ${1}
      read res
      if [ "${res}" = "" -o "`DownCase ${res}`" = "y" ]; then #{{{
        stop_agent=1
      fi #}}}
    fi #}}}
  fi #}}}

  if [ ${stop_agent} -eq 1 ]; then #{{{
    StopAgent
  else
    Error "${1} is currently running. Terminate it and run uninstall.sh again."
    exit 5
  fi #}}}
} #}}}

IsRunning() #{{{
{
  for program in ${*}; do #{{{
    if [ `TestRunning ${program}` -eq 0 ]; then #{{{
      RequestShutdown ${program}
    fi #}}}
  done #}}}
} #}}}

CheckRunning() #{{{
{
  if [ `TestRunning VV` -eq 0 ]; then #{{{
    Error "A backup, restore, or sync is ongoing. Cannot uninstall."
    exit 5
  fi #}}}

  if [ ${force} -eq 1 ]; then #{{{
    LogMessage "Stopping the Agent daemons for uninstall."
    ${init_dir}/vvagent stop-all quiet

    retry=5
    burun=`TestRunning buagent`
    vvrun=`TestRunning VVAgent`
    while [ ${retry} -gt 0 ]; do #{{{
      [ ${burun} -eq 1 -a ${vvrun} -eq 1 ] && break
      sleep 2
      retry=`expr ${retry} - 1`
      burun=`TestRunning buagent`
      vvrun=`TestRunning VVAgent`
    done #}}}
    return
  fi #}}}

  IsRunning ${progVVAgent} ${progVV}
  [ ${with_buagent} -eq 1 ] && IsRunning ${progbuagent}
} #}}}

Usage() #{{{
{
  cat <<EOS
Usage: ${bn0} [options]

    -force | -F             Forces the uninstallation. Will stop the Agent
                            daemons. Uninstallation will fail if a backup,
                            restore, or synchronization operation is currently
                            active. Without -full, will only remove the
                            program files.
    -full | -f              Performs a full uninstallation (removes the
                            configuration files).
    -quiet | -Q             Performs a quiet uninstall. Does not echo output to
                            the screen.
    -log=NAME | -L=NAME     Writes the log to the specified FILE.
    -help                   Shows this text.

EOS
} #}}}

ParamName() #{{{
{
  echo ${1} | cut -f1 -d=
} #}}}

ParamValue() #{{{
{
  echo ${1} | cut -f2 -d=
} #}}}

Setup() #{{{
{
  binsh=`which sh`
  binawk=`which awk`
  sh_ext=so

  os_name=`uname -s`
  rc_root=/etc

  case ${os_name} in #{{{
    AIX)    init_dir=/etc/rc.d
            run_levels=2
            ;;
    HP-UX)  init_dir=/sbin/init.d
            rc_root=/sbin
            sh_ext=sl
            platform=HPUX
            ;;
    Linux)  if [ -f /etc/rc.d/init.d/functions ]; then #{{{ Red Hat
              init_dir=/etc/rc.d/init.d
            elif [ -f /etc/rc.status ]; then # SUSE
              init_dir=/etc/init.d
            elif [ -f /lib/lsb/init-functions ]; then # Debian/Ubuntu
              init_dir=/etc/init.d
            else # Other
              init_dir=/etc/rc.d
            fi #}}}

            # VMware console is Linux based, so figure out if we
            # are on VMware console by checking for a special file
            if [ -f /etc/vmware-release ]; then 
              platform=Linux-ESX
            fi
            ;;
    SunOS)  init_dir=/etc/init.d
            ;;
  esac #}}}

  [ -z "${platform}" ] && platform=${os_name}
} #}}}

# Daemons return 1 if not running, and 0 if running
TestRunning() #{{{
{
  case ${1} in #{{{
    VVAgent)  ${agentDir}/VVAgent -S > /dev/null 2>&1 
              RC=${?}
              ;;
    buagent)  ${agentDir}/buagent --status > /dev/null 2>&1 
              RC=${?}
              ;;
    VV)       res=`ps -e | grep "${1}" | grep -v VVAgent | grep -v grep | wc -l`
              [ ${res} -eq 1 ] && RC=0 || RC=1
              ;;
    *)        RC=1
  esac #}}}
  echo ${RC}
  return ${RC}
} #}}}

FirewallError() #{{{
{
  LogMessage <<EOS
The firewall configuration on this ESX server could not be modified to close
the ports the Agent for VMware ESX Server needed for configuration and backups
on the default ports.

EOS

  if [ ${#} -ne 0 ]; then #{{{
    LogMessage <<EOS
Firewall configuration could not be completed because:
    ${@}
EOS
  else
    LogMessage "Firewall configuration could not be completed for an unknown reason."
  fi #}}}
} #}}}

AdjustFirewall() #{{{
{
  FW=/usr/sbin/esxcfg-firewall
  if [ ! -x ${FW} ]; then #{{{
    FirewallError "The firewall configuration program (${FW}) could not be found."
    return 1
  fi #}}}

  ${FW} -q ${fwServiceName} 2> /dev/null > /dev/null
  if [ ${?} -eq 243 ]; then #{{{
    FirewallError "The firewall service ${fwServiceName} could not be found."
    return 1
  fi #}}}

  ${FW} -d ${fwServiceName} 2> /dev/null > /dev/null
  if [ ${?} -ne 0 ]; then #{{{
    FirewallError
    return 1
  fi #}}}

  ${FW} -q ${fwServiceName} 2> /dev/null > /dev/null
  if [ ${?} -ne 1 ]; then #{{{
    FirewallError
    return 1
  fi #}}}

  LogMessage <<EOS

The firewall configuration on this ESX server has been modified to close the required Agent ports.

EOS
} #}}}

ResetESXPorts() #{{{
{
    fwServiceName=evaultAgent
    AdjustFirewall
} #}}}

UninstallRCScript() #{{{
{
  [ -f "${init_dir}/vvagent" ] || return

  r1=`grep -c "agentDir=\"${dn0}\"" "${init_dir}/vvagent"`
  r2=`grep -c "agentDir='${dn0}'" "${init_dir}/vvagent"`
  r3=`grep -c "agentDir=${dn0}" "${init_dir}/vvagent"`
  refers=`expr ${r1} + ${r2} + ${r3}`

  if [ ${refers} -eq 0 ]; then #{{{
    LogMessage <<EOS
Not removing runlevel control scripts; they do not refer to this Agent
installation.
EOS
    return
  fi #}}}

  removed=0

  case ${platform} in 
    Linux*) if [ -x /sbin/chkconfig ]; then #{{{
               /sbin/chkconfig --del vvagent 2> /dev/null > /dev/null
               removed=1
            elif [ -x /usr/sbin/update-rc.d ]; then
               /usr/sbin/update-rc.d vvagent remove > /dev/null 2> /dev/null
               removed=1
            fi #}}}
  esac

  if [ ${removed} -eq 0 ]; then #{{{
    files=`find ${rc_root} -name [SK][90]5vvagent`
    rm -f ${files}
  fi #}}}

  rm -f ${init_dir}/vvagent
} #}}}

UninstallInittab() #{{{
{
  [ -r /etc/inittab -a -w /etc/inittab ] || return

  # Do not print a message if there's no reference to VVAgent in the inittab at
  # all.
  refers=`grep -c VVAgent /etc/inittab`
  [ ${refers} -eq 0 ] && return

  refers=`grep -c "${dn0}" /etc/inittab`

  if [ ${refers} -eq 0 ]; then #{{{
    LogMessage "Not modifying /etc/inittab; it does not refer to this Agent installation."
    return
  fi #}}}

  LogMessage <<EOS
Removing references to VVAgent from /etc/inittab ...
Backed up to /etc/inittab.${uninstall_time}
EOS

  cp -p /etc/inittab /etc/inittab.${uninstall_time} 2> /dev/null > /dev/null
  grep -v ${progVVAgent} /etc/inittab > /etc/inittab.tmp
  mv -f /etc/inittab.tmp /etc/inittab 2> /dev/null > /dev/null
} #}}}

Initialize() #{{{
{
  Setup

  uninstall_time=`date +%Y%m%d%H%M%S`
  logfile=

  quiet=0
  force=0
  force_stop_agent=0
  full=0

  for arg in ${@}; do #{{{
    case ${arg} in #{{{
      -quiet|-Q)        quiet=1 ;;
      -force|-F)        force=1
                        force_stop_agent=1
                        ;;
      -full|-f)         full=1 ;;
      -log=*|-L=*)      newlogfile=`ParamValue ${arg}`
                        logpath=`dirname "${newlogfile}"`
                        if [ ! -d "${logpath}" ]; then
                          Error "Log path ${logpath} does not exist."
                          exit 1
                        fi
                        logfile="${newlogfile}"
                        ;;
      -reset|-r)        ResetESXPorts
                        exit 0
                        ;;
      -help|-h|-H|-[?]) Usage
                        exit 0
                        ;;
      *)                Error "Unknown parameter ${arg}."
                        Usage
                        exit 1
                        ;;
    esac #}}}
  done #}}}

  [ -z "${logfile}" ] && logfile="/tmp/Agent-Uninstall-${uninstall_time}.log"

  with_buagent=0

  progVVAgent=VVAgent
  progbuagent=buagent
  progVV=VV

  [ -x "${dn0}/${progbuagent}" ] && with_buagent=1
} #}}}

Centre() #{{{
{
  s=`echo ${@} | wc -c`
  s=`expr ${s}`
  if [ ${s} -ge 40 ]; then
    LogMessage "${@}"
  else
    o=`expr ${s} / 2`
    o=`expr 40 - ${o}`

    LogMessage "`printf \"%${o}s%s\" \" \" \"${@}\"`"
  fi
} #}}}

UninstallBanner() #{{{
{
  LogMessage <<EOS


EOS
  Centre "Uninstalling `Name`"
  LogMessage <<EOS


EOS
} #}}}

Name() #{{{
{
  [ -f "${dn0}/.name" ] && cat "${dn0}/.name" || echo "Backup Agent"
} #}}}

CheckPermissions() #{{{
{
  ok=0

  [ -w ${init_dir} ] || ok=1

  if [ -r /etc/inittab ]; then #{{{
    [ -w /etc/inittab ] || ok=1
  fi #}}}

  if [ ${ok} -ne 0 ]; then #{{{
    Error "Insufficient privileges to uninstall Agent. Login as root and try again."
    exit 1
  fi #}}}
} #}}} 

AskUninstall() #{{{
{
  [ ${force} -eq 1 ] && return

  asked=0
  isok=0
  printf "Do you really wish to proceed with the uninstall? (y/[N]) "
  read ask

  while [ ${asked} -eq 0 ]; do #{{{
    case ${ask} in #{{{
      n|N)  isok=0
            asked=1
            ;;
      y|Y)  isok=1
            asked=1
            ;;
      *)    if [ -z "${ask}" ]; then #{{{
              isok=0
              asked=1
            else
              printf "Unknown choice. Try again. (y/[N]) "
              read ask
            fi #}}}
            ;;
    esac #}}}
  done #}}}

  if [ ${isok} -eq 0 ]; then #{{{
    LogMessage "Aborting uninstall."
    exit 4
  fi #}}}
} #}}}

Initialize "${@}"
CheckPermissions

UninstallBanner

CheckRunning

LogMessage "Uninstalling Agent..."
UninstallInittab
UninstallRCScript

if [ "${platform}" = "Linux-ESX" ]; then #{{{
  ResetESXPorts
fi #}}}

if [ -f "${dn0}/.installed" -a -f "${dn0}/.installed.gz" ]; then
   rm -f "${dn0}/.installed"
fi

if [ -f "${dn0}/.installed" -o -f "${dn0}/.installed.gz" ]; then #{{{
  [ -f "${dn0}/.installed.gz" ] && gzip -d "${dn0}/.installed.gz"

  while read file; do
    rm -f "${file}"
  done < "${dn0}/.installed"
else
  rm -f "${dn0}/AgtUpgd"
  rm -f "${dn0}/buagent"
  rm -f "${dn0}/VV"
  rm -f "${dn0}/VVAgent"
  rm -f "${dn0}/*.so.*"
  rm -f "${dn0}/*.so"
  rm -f "${dn0}/register"
  rm -f "${dn0}/uninstall.sh"
  rm -f "${dn0}/xlogcat"
  rm -f "${dn0}/RegPlugins"
  rm -f "${dn0}/set_language"
  rm -f "${dn0}/cleansnap"
  rm -f "${dn0}/vi3tool"
fi #}}}

rm -rf "${dn0}/Languages"
rm -f "${dn0}/.name" "${dn0}/.installed*"

if [ ${full} -eq 0 -a ${quiet} -eq 0 ]; then #{{{
  printf "Do you also want to remove the installation directory?\n"
  printf "This will remove jobs, settings, etc. (y/[N]) "
  read ask
  asked=0

  while [ ${asked} -eq 0 ]; do #{{{
    case ${ask} in #{{{
      n|N)  full=0
            asked=1
            ;;
      y|Y)  full=1
            asked=1
            ;;
      *)    if [ -z "${ask}" ]; then #{{{
              full=0
              asked=1
            else
              printf "Unknown choice. Try again. (y/[N]) "
              read ask
            fi #}}}
            ;;
    esac #}}}
  done #}}}
fi #}}}

if [ ${full} -eq 1 ]; then #{{{
  LogMessage "Removing full installation from "${dn0}"."
  vvuninstall=/tmp/.vvuninstall.$$
  cat >> ${vvuninstall} <<EOS
#/bin/bash
cd /tmp
rm -rf "${dn0}"
exec rm -f ${vvuninstall}
EOS
else
  LogMessage "Agent data kept in "${dn0}"."
fi #}}}

LogMessage <<EOS

Agent successfully uninstalled.
EOS

if [ "${platform}" = "HPUX" ]; then #{{{
  rm -f /var/tmp/sh$$.*
fi #}}}

if [ ${full} -eq 1 ]; then #{{{
  chmod 755 ${vvuninstall}
  exec ${vvuninstall}
fi #}}}

exit 0
